<?php

namespace Unific\Extension\Helper\Data;

use Magento\Framework\Exception\InputException;
use Symfony\Component\Config\Definition\Exception\Exception;

class Order extends AbstractData
{
    protected $customerSession;
    protected $dataObjectConverter;

    // Holds a Order API DATA object
    protected $order;

    protected $returnData = array();

    /**
     * OrderPlugin constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Unific\Extension\Logger\Logger $logger
     * @param \Unific\Extension\Helper\Filter $filterHelper
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Framework\Api\ExtensibleDataObjectConverter $dataObjectConverter
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Unific\Extension\Logger\Logger $logger,
        \Unific\Extension\Helper\Filter $filterHelper,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\Api\ExtensibleDataObjectConverter $dataObjectConverter
    )
    {
        $this->customerSession = $customerSession;
        $this->dataObjectConverter = $dataObjectConverter;

        parent::__construct($context, $logger, $filterHelper);
    }

    /**
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     */
    public function setOrder(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        $this->order = $order;
        $this->setOrderInfo();
    }

    /**
     * @return \Magento\Sales\Model\Data\OrderInterface
     */
    public function getOrder()
    {
        return $this->order;
    }

    /**
     * @return void
     */
    protected function setOrderInfo()
    {
        $this->returnData = $this->dataObjectConverter->toFlatArray($this->order, [], \Magento\Sales\Api\Data\OrderInterface::class);

        $this->returnData['order_items'] = array();
        foreach($this->order->getItems() as $item)
        {
            $itemData = $this->dataObjectConverter->toFlatArray($item, [], \Magento\Sales\Api\Data\OrderItemInterface::class);
            $itemData['free_shipping'] = (isset($itemData['free_shipping']) && $itemData['free_shipping'] == true) ? 1 : 0;

            if(isset($itemData['is_qty_decimal']))
            {
                $itemData['is_qty_decimal'] = (int)$itemData['is_qty_decimal'];
            }

            if(isset($itemData['qty_ordered']))
            {
                $itemData['qty_ordered'] = (int)$itemData['qty_ordered'];
            }

            if(isset($itemData['qty_canceled']))
            {
                $itemData['qty_canceled'] = (int)$itemData['qty_canceled'];
            }

            if(isset($itemData['qty_invoiced']))
            {
                $itemData['qty_invoiced'] = (int)$itemData['qty_invoiced'];
            }

            if(isset($itemData['qty_refunded']))
            {
                $itemData['qty_refunded'] = (int)$itemData['qty_refunded'];
            }

            if(isset($itemData['qty_shipped']))
            {
                $itemData['qty_shipped'] = (int)$itemData['qty_shipped'];

            }

            $this->returnData['order_items'][] = $itemData;
        }

        $this->returnData['addresses'] = array();
        $this->returnData['addresses']['billing'] = $this->dataObjectConverter->toFlatArray($this->order->getBillingAddress(), [], \Magento\Sales\Api\Data\OrderAddressInterface::class);
        $this->returnData['addresses']['shipping'] = $this->dataObjectConverter->toFlatArray($this->order->getShippingAddress(), [], \Magento\Sales\Api\Data\OrderAddressInterface::class);
        $this->returnData['payment'] = $this->dataObjectConverter->toFlatArray($this->order->getPayment(), [], \Magento\Sales\Api\Data\OrderPaymentInterface::class);
    }

    /**
     * @return array
     */
    public function getOrderInfo()
    {
        // Sanitize order
        $this->returnData = array_intersect_key($this->returnData, array_flip($this->filterHelper->getOrderWhiteList()));

        // Sanitize order payment
        $this->returnData['payment'] = array_intersect_key($this->returnData['payment'], array_flip($this->filterHelper->getPaymentWhitelist()));

        for($i=0;$i<count($this->returnData['order_items']);$i++)
        {
            $this->returnData['order_items'][$i] = array_intersect_key($this->returnData['order_items'][$i], array_flip($this->filterHelper->getItemsWhitelist()));
        }

        return $this->filterHelper->sanitizeAddressData($this->returnData);
    }
}
