<?php

namespace Unific\Extension\Plugin;

class CategoryPlugin extends AbstractPlugin
{
    protected $categoryDataHelper;

    /**
     * @param \Unific\Extension\Logger\Logger $logger
     * @param \Unific\Extension\Connection\Rest\Connection $restConnection
     * @param \Magento\Framework\DataObjectFactory $dataObjectFactory
     * @param \Unific\Extension\Helper\Filter $filterHelper
     * @param \Unific\Extension\Helper\Data\Category $categoryDataHelper
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        \Unific\Extension\Logger\Logger $logger,
        \Unific\Extension\Connection\Rest\Connection $restConnection,
        \Magento\Framework\DataObjectFactory $dataObjectFactory,
        \Unific\Extension\Helper\Filter $filterHelper,
        \Unific\Extension\Helper\Data\Category $categoryDataHelper,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    )
    {
        parent::__construct(
            $logger,
            $restConnection,
            $dataObjectFactory,
            $filterHelper,
            $scopeConfig
        );

        $this->categoryDataHelper = $categoryDataHelper;
    }

    /**
     * @param \Magento\Catalog\Model\Category $subject
     * @return \Magento\Catalog\Model\Category
     */
    public function afterSave(\Magento\Catalog\Model\Category $subject)
    {
        if($this->scopeConfig->getValue('unific/webhook/category')) {
            $this->setEntity('category');

            if($subject->getCreatedAt() != $subject->getUpdatedAt() && $subject->getUpdatedAt() != null)
            {
                $this->setAction('update');
            }

            $this->categoryDataHelper->setCategory($subject);
            $this->processWebhook($this->categoryDataHelper->getCategoryInfo());
        }

        return $subject;
    }
}
