<?php

namespace Unific\Extension\Plugin;

class GuestCartPlugin extends AbstractPlugin
{
    protected $cartDataHelper;
    protected $cartRepository;
    protected $quoteIdMaskFactory;
    protected $customerSession;
    protected $customerDataHelper;

    /**
     * @param \Unific\Extension\Logger\Logger $logger
     * @param \Unific\Extension\Connection\Rest\Connection $restConnection
     * @param \Magento\Framework\DataObjectFactory $dataObjectFactory
     * @param \Unific\Extension\Helper\Filter $filterHelper
     * @param \Unific\Extension\Helper\Data\Cart $cartDataHelper
     * @param \Unific\Extension\Helper\Data\Customer $customerDataHelper
     * @param \Magento\Quote\Api\CartRepositoryInterface $cartRepository
     * @param \Magento\Quote\Model\QuoteIdMaskFactory $quoteIdMaskFactory
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Customer\Model\Session $customerSession
     */
    public function __construct(
        \Unific\Extension\Logger\Logger $logger,
        \Unific\Extension\Connection\Rest\Connection $restConnection,
        \Magento\Framework\DataObjectFactory $dataObjectFactory,
        \Unific\Extension\Helper\Filter $filterHelper,
        \Unific\Extension\Helper\Data\Cart $cartDataHelper,
        \Unific\Extension\Helper\Data\Customer $customerDataHelper,
        \Magento\Quote\Api\CartRepositoryInterface $cartRepository,
        \Magento\Quote\Model\QuoteIdMaskFactory $quoteIdMaskFactory,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Customer\Model\Session $customerSession
    )
    {
        parent::__construct(
            $logger,
            $restConnection,
            $dataObjectFactory,
            $filterHelper,
            $scopeConfig
        );

        $this->cartDataHelper = $cartDataHelper;
        $this->cartRepository = $cartRepository;
        $this->quoteIdMaskFactory = $quoteIdMaskFactory;
        $this->customerSession = $customerSession;
        $this->customerDataHelper = $customerDataHelper;

        $this->setEntity('checkout');
    }

    /**
     * @param $subject
     * @param callable $proceed
     * @param $cartId
     * @param \Magento\Quote\Model\Quote\Address $address
     * @return array
     */
    public function aroundEstimateByExtendedAddress($subject, callable $proceed, $cartId, \Magento\Quote\Model\Quote\Address $address)
    {
        $returnValue = $proceed($cartId, $address);

        if($this->scopeConfig->getValue('unific/webhook/cart') && $this->customerSession->isLoggedIn() === false) {
            if (!$subject instanceof \Magento\Quote\Model\ShippingMethodManagement\Interceptor) {
                $quoteIdMask = $this->quoteIdMaskFactory->create()->load($cartId, 'masked_id');
                $this->cartDataHelper->setCart($this->cartRepository->get($quoteIdMask->getQuoteId()));

                if($address->getFirstname() != null)
                {
                    $this->cartDataHelper->setAddressData($address);

                    if ($this->cartDataHelper->getCart()->getCreatedAt() != $this->cartDataHelper->getCart()->getUpdatedAt() &&
                        $this->cartDataHelper->getCart()->getUpdatedAt() != null) {
                        $this->setAction('update');
                    }

                    $this->processWebhook($this->cartDataHelper->getCartInfo());
                }
            }
        }

        // Send customer
        if($this->scopeConfig->getValue('unific/webhook/customer') && !$subject instanceof \Magento\Quote\Model\ShippingMethodManagement\Interceptor) {
            $this->setEntity('customer');
            $this->setAction('update');

            // Send a customer update, if not logged in, generate a customer from quote data
            $quoteIdMask = $this->quoteIdMaskFactory->create()->load($cartId, 'masked_id');
            $this->customerDataHelper->generateGuestCustomer($this->cartRepository->get($quoteIdMask->getQuoteId()));

            $customerData = $this->customerDataHelper->getCustomerInfo();
            $this->processWebhook($customerData);
        }

        return $returnValue;
    }
}
