<?php

namespace Unific\Extension\Plugin;

class InvoicePlugin extends AbstractPlugin
{
    protected $orderDataHelper;
    protected $customerSession;
    protected $orderRepository;

    /**
     * @param \Unific\Extension\Logger\Logger $logger
     * @param \Unific\Extension\Connection\Rest\Connection $restConnection
     * @param \Magento\Framework\DataObjectFactory $dataObjectFactory
     * @param \Unific\Extension\Helper\Filter $filterHelper
     * @param \Unific\Extension\Helper\Data\Order $orderDataHelper
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     */
    public function __construct(
        \Unific\Extension\Logger\Logger $logger,
        \Unific\Extension\Connection\Rest\Connection $restConnection,
        \Magento\Framework\DataObjectFactory $dataObjectFactory,
        \Unific\Extension\Helper\Filter $filterHelper,
        \Unific\Extension\Helper\Data\Order $orderDataHelper,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
    )
    {
        parent::__construct(
            $logger,
            $restConnection,
            $dataObjectFactory,
            $filterHelper,
            $scopeConfig
        );

        $this->orderDataHelper = $orderDataHelper;
        $this->customerSession = $customerSession;
        $this->orderRepository = $orderRepository;

        $this->setEntity('order');
    }

    /**
     * @param $subject
     * @return mixed
     */
    public function afterRegister($subject)
    {
        $this->setAction('invoice');

        if($this->scopeConfig->getValue('unific/webhook/order')) {
            $this->orderDataHelper->setOrder($this->orderRepository->get($subject->getOrder()->getId()));
            $this->setAction('invoice');
            $this->processWebhook($this->orderDataHelper->getOrderInfo());
        }

        return $subject;
    }
}
