<?php

namespace Unific\Extension\Setup;

use Magento\Framework\Setup\InstallSchemaInterface;

use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
use Magento\Framework\DB\Adapter\AdapterInterface;

class InstallSchema implements InstallSchemaInterface
{
    public function install(SchemaSetupInterface $setup,
                            ModuleContextInterface $context)
    {
        $installer = $setup;
        $installer->startSetup();

        /**
         * The MySQL Table to implement the AMQP behaviour
         */
        $table = $installer->getConnection()->newTable($installer->getTable('unific_extension_message_queue'))
            ->addColumn('guid', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 0, array(
                'nullable' => true,
            ), 'A unique GUID identifier')
            ->addColumn('message', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 0, array(
                'nullable' => true,
            ), 'Message')
            ->addColumn('headers', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 0, array(
                'nullable' => false,
            ), 'Message')
            ->addColumn('url', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 0, array(
                'nullable' => false,
            ), 'Message')
            ->addColumn('historical', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 0, array(
                'nullable' => false,
                'default' => 0
            ), 'Message')
            ->addColumn('request_type', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 0, array(
                'nullable' => false,
            ), 'The request type, can be POST, PUT, DELETE')
            ->addColumn('retry_amount', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 0, array(
                'unsigned' => true,
                'nullable' => false,
                'default' => 0
            ), 'Group ID')
            ->addColumn('max_retry_amount', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 0, array(
                'unsigned' => true,
                'nullable' => false,
                'default' => 20
            ), 'Group ID')
            ->addColumn('response_error', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 0, array(
                'nullable' => true,
            ), 'Message')
            ->addColumn('response_http_code', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 0, array(
                'unsigned' => true,
                'nullable' => false,
            ), 'Group ID')
            ->addColumn('request_date_first', \Magento\Framework\DB\Ddl\Table::TYPE_DATE, 0, array(
                'nullable' => true,
            ), 'Date where this request was first sent')
            ->addColumn('request_date_last', \Magento\Framework\DB\Ddl\Table::TYPE_DATE, 0, array(
                'nullable' => true,
            ), 'Date where this request was last sent')
            ->addColumn(
                'request_date_first',
                \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP,
                null,
                ['nullable' => false, 'default' => \Magento\Framework\DB\Ddl\Table::TIMESTAMP_INIT],
                'Date where this request was first sent')
            ->addColumn(
                'request_date_last',
                \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP,
                null,
                ['nullable' => false, 'default' => \Magento\Framework\DB\Ddl\Table::TIMESTAMP_INIT_UPDATE],
                'Date where this request was last sent')
            ->setComment(
                'Message Queue'
            );

        $installer->getConnection()->createTable($table);

        /**
         * The audit log for just about anything that the extension does
         */
        $table = $installer->getConnection()->newTable($installer->getTable('unific_extension_audit_log'))
            ->addColumn('id', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, null, array(
                'identity' => true,
                'unsigned' => true,
                'nullable' => false,
                'primary' => true,
            ), 'ID')
            ->addColumn('log_guid', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 0, array(
                'nullable' => true,
            ), 'A unique GUID identifier')
            ->addColumn('log_error_type', \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER, 0, array(
                'unsigned' => true,
                'nullable' => false,
            ), 'The type of the error, constants are in the log helper')
            ->addColumn('log_action_parameters', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 0, array(
                'nullable' => true,
            ), 'The type of action performed, this can be a request, response or reconfiguration')
            ->addColumn('log_action_payload', \Magento\Framework\DB\Ddl\Table::TYPE_TEXT, 0, array(
                'nullable' => true,
            ), 'The payload of the action performed')
            ->addColumn(
                'created_at',
                \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP,
                null,
                ['nullable' => false, 'default' => \Magento\Framework\DB\Ddl\Table::TIMESTAMP_INIT],
                'Creation date')
            ->addColumn(
                'updated_at',
                \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP,
                null,
                ['nullable' => false, 'default' => \Magento\Framework\DB\Ddl\Table::TIMESTAMP_INIT_UPDATE],
                'Update date')
            ->setComment(
                'Unific Audit Log'
            );

        $installer->getConnection()->createTable($table);

        $installer->endSetup();
    }
}